from lxml import etree
from lxml.builder import E


class Tree:
    def __init__(self, data):
        self.instance = etree.fromstring(data)

    def __add__(self, other):
        self.instance.append(other)
        return self
        
    def __bytes__(self):
        return etree.tostring(self.instance)


# root = E.root(
#     E.child1("Content for child 1"),
#     E.child2("Content for child 2")
# )


# a = etree.tostring(root)
# print(a)
# tree = Tree(a)
# print(bytes(tree))
# tree += E.haha(id='12')
# print(bytes(tree))
# # print(type(tree))





from zipfile import ZipFile, ZIP_DEFLATED
from io import BytesIO


class FakeZip(object):
    """伪Zip对象，只作文件内容存储
    解决Zip无法直接替换文件问题
    """
    def __init__(self, file_path):
        self._dict = {}
        if isinstance(file_path, bytes):
            zip = ZipFile(BytesIO(file_path), 'r')
        else:
            zip = ZipFile(file_path)
        for fileinfo in zip.infolist():
            file_data = zip.open(fileinfo).read()
            self._dict[fileinfo.filename] = file_data

    def __getitem__(self, filename):
        return self._dict.get(filename)

    def __setitem__(self, filename, content):
        self._dict[filename] = content  

    def __bytes__(self):
        with BytesIO() as f:
            self.write(f)
            return f.getvalue()

    def write(self, f):
        """写入到文件"""
        with ZipFile(f, mode='w', compression=ZIP_DEFLATED) as zf:
            for k, v in self._dict.items():
                zf.writestr(k, v)

    def save(self, path):
        """保存为文件"""
        with open(path, 'wb') as f:
            self.write(f)





import pytest

from ..src.wordx.fake_zip import FakeZip




# class TestFakeZip:
#     @pytest.fixture
#     def fake_zip(self):
#         zip_data = BytesIO()
#         with ZipFile(zip_data, mode='w', compression=ZIP_DEFLATED) as zf:
#             zf.writestr('file1.txt', 'content1')
#             zf.writestr('file2.txt', 'content2')
#         zip_data.seek(0)
#         return FakeZip(zip_data)

#     def test_get_existing_file(self, fake_zip):
#         content = fake_zip.get('file1.txt')
#         assert content == b'content1'

#     def test_get_non_existing_file(self, fake_zip):
#         content = fake_zip.get('file3.txt')
#         assert content is None

#     def test_replace_existing_file(self, fake_zip):
#         fake_zip.replace('file1.txt', 'new_content')
#         content = fake_zip.get('file1.txt')
#         assert content == b'new_content'

#     def test_replace_non_existing_file(self, fake_zip):
#         fake_zip.replace('file3.txt', 'new_content')
#         content = fake_zip.get('file3.txt')
#         assert content == b'new_content'

#     def test_add_new_file(self, fake_zip):
#         fake_zip.add('file3.txt', 'new_content')
#         content = fake_zip.get('file3.txt')
#         assert content == b'new_content'

#     def test_save_zip_file(self, fake_zip, tmp_path):
#         file_path = tmp_path / 'test.zip'
#         fake_zip.save(file_path)
#         assert file_path.exists()

#     def test_bytes_conversion(self, fake_zip):
#         zip_data = bytes(fake_zip)
#         assert isinstance(zip_data, bytes)
#         assert len(zip_data) > 0


from zgui import * 
from tkinter.ttk import Notebook
from wordx.word_file import WordFile 
from utility import SuperWordFile
import sys
import time 


app =  App({'title':'WordX模板工具箱','size':(420,330),'loc':(500,300)})

root = app.instance

note1=Notebook(root) 
note1.pack(fill=BOTH,expand=True)
 
fr1=Frame(root,relief='ridge', borderwidth=1) 
fr2=Frame(root,relief='ridge', borderwidth=1)
fr3=Frame(root,relief='ridge', borderwidth=1)

note1.add(fr1,text='编辑') 
note1.add(fr2,text='测试') 

label1 = Label(fr1, text='模板目录')
label1.pack()
label1.place(x=5,y=5)

label2 = Label(fr1, text='模板文件')
label2.pack()
label2.place(x=5,y=30)

label3 = Label(fr1, text='资源文件')
label3.pack()
label3.place(x=5,y=55)

label4 = Label(fr1, text='写入路径')
label4.pack()
label4.place(x=5,y=80)

input1 = app.input(fr1, print, width=48)
input1.pack()
input1.place(x=65,y=5)

input2 = app.input(fr1, print, width=48)
input2.pack()
input2.place(x=65,y=30)

input3 = app.input(fr1, print, width=48)
input3.pack()
input3.place(x=65,y=55)

input4 = app.input(fr1, print, width=48)
input4.pack()
input4.place(x=65,y=80)


def get_edit_input():
    folder = input1.get()
    file = input2.get()
    return folder, file

def on_click_extract_document():
    folder, file = get_edit_input()
    wf = SuperWordFile(folder, file)
    wf.extract('document.xml')

def on_click_extract_header():
    folder, file = get_edit_input()
    wf = SuperWordFile(folder, file)
    wf.extract('header.xml')

def on_click_extract_footer():
    folder, file = get_edit_input()
    wf = SuperWordFile(folder, file)
    wf.extract('footer.xml')

def on_click_generate():
    folder, file = get_edit_input()
    wf = SuperWordFile(folder, file)
    wf.generate()

def on_click_test():
    folder, file = get_edit_input()
    wf = SuperWordFile(folder, file)
    wf.test()

def on_click_write():
    folder = input1.get()
    tpl_file = input2.get()
    res_file = input3.get()
    res_path = input4.get()
    wf = SuperWordFile(folder, tpl_file)
    wf.write(res_file, res_path)

def on_click_render():
    folder, file = get_edit_input()
    wf = SuperWordFile(folder, file)
    wf.render()

def on_click_word2xml_h():
    SuperWordFile.word2xml('h')

def on_click_word2xml_v():
    SuperWordFile.word2xml('v')

def on_click_xml2word_h():
    SuperWordFile.xml2word('h')

def on_click_xml2word_v():
    SuperWordFile.xml2word('v')

button = app.button(fr1, '提取document', on_click_extract_document, width=13)
button.pack()
button.place(x=30,y=110)

button = app.button(fr1, '提取header', on_click_extract_header, width=13)
button.pack()
button.place(x=155,y=110)

button = app.button(fr1, '提取footer', on_click_extract_footer, width=13)
button.pack()
button.place(x=280,y=110)

button = app.button(fr1, '组装', on_click_generate, width=22)
button.pack()
button.place(x=30,y=145)

button = app.button(fr1, '测试', on_click_test, width=22)
button.pack()
button.place(x=220,y=145)

button = app.button(fr1, '渲染', on_click_render, width=49)
button.pack()
button.place(x=30,y=180)

button = app.button(fr1, '写入', on_click_write, width=49)
button.pack()
button.place(x=30,y=215)

button = app.button(fr2, 'word转xml(竖向)', on_click_word2xml_v, width=49)
button.pack()
button.place(x=30,y=30)

button = app.button(fr2, 'word转xml(横向)', on_click_word2xml_h, width=49)
button.pack()
button.place(x=30,y=80)

button = app.button(fr2, 'xml转word(竖向)', on_click_xml2word_v, width=49)
button.pack()
button.place(x=30,y=130)

button = app.button(fr2, 'xml转word(横向)', on_click_xml2word_h, width=49)
button.pack()
button.place(x=30,y=180)

app.run()


from wordx.utils.tree import Tree, E
import random


class ResourceMixin:
    def get_resource(self, res_path):
        return self[f'word/{res_path}']

    def add_resource(res_path, res_bytes):
        self[f'word/{res_path}'] = res_bytes

    def get_document(self):
        return self[f'word/document.xml']

    def save_resource(self, res_path, file_path):
        with open(file_path, 'wb') as f:
            res_bytes = self.get_resource(res_path)
            f.write(res_bytes)


class RelationMixin:
    def get_relations(self, xml_file):
        return self[f'word/_rels/{xml_file}.rels']

    def add_relations(self, xml_file, relations):
        self[f'word/_rels/{xml_file}.rels'] = relations

    def merge_relations(self, relations_a, relations_b):
        relation_tree = etree.fromstring(relations_a)
        for relation in relations_b:
            relation_element = E.Relationship(
                Id=relation['id'], 
                Type=relation['type'], 
                Target=relation['target'])
            relation_tree.append(relation_element)
        return etree.tostring(relation_tree)

    def add_relation(self, xml_file, relation_type, relation_target, relation_id = None):
        relations = self.get_relations(xml_file)
        if not relations:
            relations = b"""<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <Relationships xmlns="http://schemas.openxmlformats.org/package/2006/relationships"></Relationships>"""
        relation_tree = Tree(relations)
        relation_id = random.randint(1000,9999) if not relation_id else relation_id
        relation_type = f"http://schemas.openxmlformats.org/officeDocument/2006/relationships/{relation_type}"
        relation_tree +=  E.Relationship(Id=f'rId{relation_id}', Type=relation_type, Target=relation_target)
        self.add_relations(xml_file, bytes(relation_tree))
        return relation_id

    def add_footer_relation(self):
        footer_relation_id = random.randint(1000,9999)
        footer_file = f'footer{footer_relation_id}.xml'
        self.add_relation('document.xml', 'footer', footer_file, footer_relation_id)
        return f'rId{footer_relation_id}', footer_file

    def add_header_relation(self):
        header_relation_id = random.randint(1000,9999)
        header_file = f'header{header_relation_id}.xml'
        self.add_relation('document.xml', 'header', header_file, header_relation_id)
        return f'rId{header_relation_id}', header_file



from wordx.word_file import WordFile 
from wordx.sheet import Sheet
import os
import json
import time
import sys


class SuperWordFile(WordFile):
    testing_word = ''  
    testing_xml = ''  

    def __init__(self, folder, file):
        path = os.path.join(folder, file)
        super().__init__(path)
        self.folder = folder

    @staticmethod
    def timestamp():
        return str(int(time.time()))

    def super_replace(self, file):
        path = os.path.join(self.folder, file)
        if os.path.exists(path):
            with open(path, 'rb') as f:
                file_bytes = f.read()
                self[f'word/{file}'] = file_bytes

    def super_save(self, file):
        path = os.path.join(self.folder, file)
        self.save(path)

    def generate(self):
        for xml_file in ['header.xml', 'footer.xml', 'document.xml']:
            self.super_replace(xml_file)
        dst_file = f'template{self.timestamp()}.docx'
        self.super_save(dst_file)
        return dst_file
        
    def test(self):
        self.render()
        # template_file = self.generate()
        # python_file_path = os.path.join(self.folder, 'test.py')
        # os.system(f'python {python_file_path} {template_file}')

    def render(self):
        python_file_path = os.path.join(self.folder, 'test.py')
        print(f'python {python_file_path}')
        os.system(f'python {python_file_path}')

    def extract(self, res_path):
        dst_path = os.path.join(self.folder, f'{res_path}')
        dst_path = os.path.join(self.folder, f'{self.timestamp()}{res_path}') if os.path.exists(dst_path) else dst_path
        with open(dst_path, 'wb') as f:
            res_bytes = self.get_resource(res_path)
            f.write(res_bytes)

    def write(self, res_file, res_path):
        res_path_ = os.path.join(self.folder, res_file)
        with open(res_path_, 'rb') as f:
            data = f.read()
            self.replace(res_path, data)
        dst_file = os.path.join(self.folder, f'{self.timestamp()}.docx')
        wf.super_save(dst_file)

    @staticmethod
    def get_resource_path(relative_path):
        if hasattr(sys, "_MEIPASS"):
            return os.path.join(sys._MEIPASS, relative_path)
        return os.path.join(relative_path)

    @classmethod
    def word2xml(cls, orientation='h'):
        if cls.testing_word:
            cls.extract_document(orientation)
        else:
            cls.new_word(orientation)

    @classmethod
    def xml2word(cls, orientation='h'):
        if cls.testing_xml:
            cls.create_word(orientation)
        else:
            cls.new_xml(orientation)

    @classmethod
    def new_word(cls, orientation):
        docx_file = 'h.docx' if orientation == 'h' else 'v.docx'
        docx_path = cls.get_resource_path(docx_file)
        save_path = f'd:/tmp/{cls.timestamp()}test.docx'
        wf = WordFile(docx_path)
        wf.save(save_path)
        os.startfile(save_path)
        cls.testing_word = save_path

    @classmethod
    def extract_document(cls, orientation):
        wf = WordFile(cls.testing_word)
        save_path = f'd:/tmp/{cls.timestamp()}document.xml'
        wf.save_resource('document.xml', save_path)
        os.startfile(save_path)
        cls.testing_word = ''

    @classmethod
    def new_xml(cls, orientation):
        docx_file = 'h.docx' if orientation == 'h' else 'v.docx'
        docx_path = cls.get_resource_path(docx_file)
        wf = WordFile(docx_path)
        save_path = f'd:/tmp/{cls.timestamp()}document.xml'
        wf.save_resource('document.xml', save_path)
        os.startfile(save_path)
        cls.testing_xml = save_path 

    @classmethod
    def create_word(cls, orientation):
        docx_file = 'h.docx' if orientation == 'h' else 'v.docx'
        docx_path = cls.get_resource_path(docx_file)
        wf = WordFile(docx_path)
        with open(cls.testing_xml, 'rb') as f:
            content = f.read()
            wf['word/document.xml'] = content
            save_path = f'd:/tmp/{cls.timestamp()}test.docx'
            wf.save(save_path)
            os.startfile(save_path)
            cls.testing_xml = ''

    

    
    
if __name__ == '__main__':
    wf = SuperWordFile(r'd:\git\lims\template\inspect\record - 副本','template.docx')
    wf.extract('document.xml')
    exit()
    wf = SuperWordFile('.','123.docx')
    # wf.extract('document.xml')
    # wf.write('document.xml', 'document2.xml')